﻿using System;
using TweetinCore.Interfaces.DTO;

namespace TweetinCore.Interfaces
{
    /// <summary>
    /// Message that can be sent privately between Twitter users
    /// </summary>
    public interface IMessage : ITwitterObject, IEquatable<IMessage>
    {
        IMessageDTO MessageDTO { get; set; }

        bool IsMessagePublished { get; }
        bool IsMessageDestroyed { get; }

        /// <summary>
        /// Id of the Message
        /// </summary>
        long Id { get; }

        /// <summary>
        /// Text contained in the message
        /// </summary>
        string Text { get; }

        /// <summary>
        /// Creation date of the message
        /// </summary>
        DateTime CreatedAt { get; }

        /// <summary>
        /// User who sent the message
        /// </summary>
        IUser Sender { get; }

        /// <summary>
        /// Receiver of the message
        /// </summary>
        IUser Receiver { get; }

        bool Publish();
        bool PublishTo(IUser recipient);
        bool Destroy();

        void SetRecipient(IUser recipient);
    }
}
