﻿using System.Collections.Generic;
using Newtonsoft.Json;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using Tweetinvi.DTO;
using Tweetinvi.Model;
using Tweetinvi.TwitterEntities;

namespace Tweetinvi.JsonConverters
{
    /// <summary>
    /// Repository of multiple converters required to generate DTO classes.
    /// </summary>
    public class JsonPropertiesConverterRepository
    {
        public static JsonConverter[] Converters { get; private set; }

        static JsonPropertiesConverterRepository()
        {
            Initialize();
        }

        private static void Initialize()
        {
            List<JsonConverter> converters = new List<JsonConverter>
            {
                new JsonTwitterObjectConverter<ITweetDTO, TweetDTO>(),
                new JsonTwitterObjectConverter<IOEmbedTweetDTO, OEmbedTweetDTO>(),
                new JsonTwitterObjectConverter<IUserDTO, UserDTO>(),
                new JsonTwitterObjectConverter<IMessageDTO, MessageDTO>(),

                new JsonTwitterObjectConverter<IRelationshipDTO, RelationshipDTO>(),
                new JsonTwitterObjectConverter<IRelationshipStateDTO, RelationshipStateDTO>(),

                new JsonTwitterObjectConverter<ICoordinates, Coordinates>(),
                new JsonTwitterObjectConverter<ILocation, Location>(),

                new JsonTwitterObjectConverter<IUrlEntity, UrlEntity>(),
                new JsonTwitterObjectConverter<IHashtagEntity, HashtagEntity>(),
                new JsonTwitterObjectConverter<IMediaEntity, MediaEntity>(),
                new JsonTwitterObjectConverter<IMediaEntitySize, MediaEntitySize>(),
                new JsonTwitterObjectConverter<IUserMentionEntity, UserMentionEntity>(),
                new JsonTwitterObjectConverter<ITweetEntities, TweetEntities>(),

                new JsonTwitterObjectConverter<IRelationship, Relationship>(),
                new JsonTwitterObjectConverter<IRelationshipState, RelationshipState>()
            };

            Converters = converters.ToArray();
        }
    }
}
