﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;

namespace Tweetinvi.JsonConverters
{
    internal class JsonTwitterIEnumerableConverter<TYPE_OUT, CONCRETE_TYPE, OUT> : JsonConverter

            where CONCRETE_TYPE : TYPE_OUT
            where OUT : IEnumerable<TYPE_OUT>
    {
        private readonly Func<IEnumerable<TYPE_OUT>, OUT> _castAction;

        public JsonTwitterIEnumerableConverter(Func<IEnumerable<TYPE_OUT>, OUT> castAction)
        {
            _castAction = castAction;
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var objectList = serializer.Deserialize<IEnumerable<CONCRETE_TYPE>>(reader);

            if (_castAction == null)
            {
                return (IEnumerable<TYPE_OUT>) objectList;
            }

            return _castAction((IEnumerable<TYPE_OUT>)objectList);
        }

        public override bool CanConvert(Type objectType)
        {
            return (objectType == typeof(TYPE_OUT));
        }

        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            throw new NotImplementedException();
        }
    }
}
