﻿using System.Collections.Generic;
using Microsoft.Practices.Unity;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;

namespace TweetinviBootstrapper.Tweet
{
    public static class TweetController
    {
        private static readonly ITweetController _tweetController;

        static TweetController()
        {
            var containter = TweetinviFactoriesBootstrapper.Container;

            _tweetController = containter.Resolve<ITweetController>();
        }

        // Publish Tweet
        public static ITweet PublishTweet(ITweet tweet)
        {
            return PublishTweet(tweet.TweetDTO);
        }

        public static ITweet PublishTweet(ITweetDTO tweetDTO)
        {
            return _tweetController.PublishTweet(tweetDTO);
        }

        // Publish TweetInReplyTo
        public static ITweet PublishTweetInReplyToTweet(ITweet tweet, ITweet tweetToReplyTo)
        {
            return PublishTweetInReplyToTweet(tweet.TweetDTO, tweetToReplyTo.TweetDTO);
        }

        public static ITweet PublishTweetInReplyToTweet(ITweetDTO tweetDTO, ITweetDTO tweetDTOToReplyTo)
        {
            return _tweetController.PublishTweetInReplyTo(tweetDTO, tweetDTOToReplyTo);
        }

        public static ITweet PublishTweetInReplyTo(ITweetDTO tweet, long tweetIdToRespondTo)
        {
            return _tweetController.PublishTweetInReplyTo(tweet, tweetIdToRespondTo);
        }

        // Publish Retweet
        public static ITweet PublishRetweet(ITweet tweet)
        {
            return _tweetController.PublishRetweet(tweet.TweetDTO);
        }

        public static ITweet PublishRetweet(ITweetDTO tweetDTO)
        {
            return _tweetController.PublishRetweet(tweetDTO);
        }

        public static ITweet PublishRetweet(long tweetId)
        {
            return _tweetController.PublishRetweet(tweetId);
        }
        
        // Get Retweet
        public static List<ITweet> GetRetweets(ITweet tweet)
        {
            return _tweetController.GetRetweets(tweet);
        }

        public static List<ITweet> GetRetweets(ITweetDTO tweetDTO)
        {
            return _tweetController.GetRetweets(tweetDTO);
        }

        public static List<ITweet> GetRetweets(long tweetId)
        {
            return _tweetController.GetRetweets(tweetId);
        }

        // Destroy Tweet
        public static bool DestroyTweet(ITweet tweet)
        {
            return _tweetController.DestroyTweet(tweet.TweetDTO);
        }

        public static bool DestroyTweet(ITweetDTO tweetDTO)
        {
            return _tweetController.DestroyTweet(tweetDTO);
        }

        public static bool DestroyTweet(long tweetId)
        {
            return _tweetController.DestroyTweet(tweetId);
        }
    }
}
