﻿using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviControllers.Direct_Messages
{
    public class MessageController : IMessageController
    {
        private readonly IMessageQueryExecutor _messageQueryExecutor;
        private readonly IMessageFactory _messageFactory;

        public MessageController(
            IMessageQueryExecutor messageQueryExecutor,
            IMessageFactory messageFactory)
        {
            _messageQueryExecutor = messageQueryExecutor;
            _messageFactory = messageFactory;
        }

        public List<IMessage> GetLatestMessagesReceived(int maximumMessages = 40)
        {
            var messagesDTO = _messageQueryExecutor.GetLatestMessagesReceived(maximumMessages);
            return _messageFactory.GenerateMessagesFromMessagesDTO(messagesDTO);
        }

        public List<IMessage> GetLatestMessagesSent(int maximumMessages = 40)
        {
            var messagesDTO = _messageQueryExecutor.GetLatestMessagesSent(maximumMessages);
            return _messageFactory.GenerateMessagesFromMessagesDTO(messagesDTO);
        }

        // Publish Message
        public IMessage PublishMessage(IMessageDTO messageDTO)
        {
            var publishedMessageDTO = _messageQueryExecutor.PublishMessage(messageDTO);
            return _messageFactory.GenerateMessageFromMessageDTO(publishedMessageDTO);
        }

        public IMessage PublishMessage(string text, IUser targetUser)
        {
            if (targetUser == null)
            {
                return null;
            }

            return PublishMessage(text, targetUser);
        }

        public IMessage PublishMessage(string text, IUserDTO targetUserDTO)
        {
            var publishedMessageDTO = _messageQueryExecutor.PublishMessage(text, targetUserDTO);
            return _messageFactory.GenerateMessageFromMessageDTO(publishedMessageDTO);
        }

        public IMessage PublishMessage(string text, long targetUserId)
        {
            var publishedMessageDTO = _messageQueryExecutor.PublishMessage(text, targetUserId);
            return _messageFactory.GenerateMessageFromMessageDTO(publishedMessageDTO);
        }

        public IMessage PublishMessage(string text, string targetUserScreenName)
        {
            var publishedMessageDTO = _messageQueryExecutor.PublishMessage(text, targetUserScreenName);
            return _messageFactory.GenerateMessageFromMessageDTO(publishedMessageDTO);
        }

        // Destroy Message
        public bool DestroyMessage(IMessage message)
        {
            if (message == null)
            {
                return false;
            }

            return DestroyMessage(message.MessageDTO);
        }

        public bool DestroyMessage(IMessageDTO messageDTO)
        {
            if (!IsMessageCurrentlyPublished(messageDTO))
            {
                return false;
            }
            
            messageDTO.IsMessageDestroyed = _messageQueryExecutor.DestroyMessage(messageDTO);
            return messageDTO.IsMessageDestroyed;
        }

        public bool DestroyMessage(long messageId)
        {
            return _messageQueryExecutor.DestroyMessage(messageId);
        }

        // Helpers
        private bool IsMessageCurrentlyPublished(IMessageDTO messageDTO)
        {
            return messageDTO != null &&
                   messageDTO.Id != -1 &&
                   messageDTO.IsMessagePublished &&
                   !messageDTO.IsMessageDestroyed;
        }
    }
}
