﻿using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviControllers.Friendship
{
    public class FriendshipController : IFriendshipController
    {
        private readonly IFriendshipQueryExecutor _friendshipQueryExecutor;
        private readonly IFriendshipFactory _friendshipFactory;
        private readonly IUserFactory _userFactory;

        public FriendshipController(
            IFriendshipQueryExecutor friendshipQueryExecutor,
            IFriendshipFactory friendshipFactory,
            IUserFactory userFactory)
        {
            _friendshipQueryExecutor = friendshipQueryExecutor;
            _friendshipFactory = friendshipFactory;
            _userFactory = userFactory;
        }

        // Get Users Requesting Friendship
        public List<long> GetUserIdsRequestingFriendship()
        {
            return _friendshipQueryExecutor.GetUserIdsRequestingFriendship();
        }

        public List<IUser> GetUsersRequestingFriendship()
        {
            var userIds = GetUserIdsRequestingFriendship();
            return _userFactory.GetUsersFromIds(userIds);
        }

        // Get Users You requested to follow
        public List<long> GetUserIdsYouRequestedToFollow()
        {
            return _friendshipQueryExecutor.GetUserIdsYouRequestedToFollow();
        }

        public List<IUser> GetUsersYouRequestedToFollow()
        {
            var userIds = GetUserIdsYouRequestedToFollow();
            return _userFactory.GetUsersFromIds(userIds);
        }

        // Create Friendship with
        public bool CreateFriendshipWith(IUser user)
        {
            if (user == null)
            {
                return false;
            }

            return CreateFriendshipWith(user.UserDTO);
        }

        public bool CreateFriendshipWith(IUserIdDTO userDTO)
        {
            return _friendshipQueryExecutor.CreateFriendshipWith(userDTO);
        }

        public bool CreateFriendshipWith(long userId)
        {
            return _friendshipQueryExecutor.CreateFriendshipWith(userId);
        }

        public bool CreateFriendshipWith(string userScreeName)
        {
            return _friendshipQueryExecutor.CreateFriendshipWith(userScreeName);
        }

        // Destroy Friendship with
        public bool DestroyFriendshipWith(IUser user)
        {
            if (user == null)
            {
                return false;
            }

            return DestroyFriendshipWith(user.UserDTO);
        }

        public bool DestroyFriendshipWith(IUserIdDTO userDTO)
        {
            return _friendshipQueryExecutor.DestroyFriendshipWith(userDTO);
        }

        public bool DestroyFriendshipWith(long userId)
        {
            return _friendshipQueryExecutor.DestroyFriendshipWith(userId);
        }

        public bool DestroyFriendshipWith(string userScreeName)
        {
            return _friendshipQueryExecutor.DestroyFriendshipWith(userScreeName);
        }

        // Update Friendship Authorizations
        public bool UpdateRelationshipAuthorizationsWith(IUser user, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            if (user == null)
            {
                return false;
            }

            return UpdateRelationshipAuthorizationsWith(user.UserDTO, retweetsEnabled, deviceNotifictionEnabled);
        }

        public bool UpdateRelationshipAuthorizationsWith(IUserIdDTO userDTO, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            return _friendshipQueryExecutor.UpdateRelationshipAuthorizationsWith(userDTO, retweetsEnabled, deviceNotifictionEnabled);
        }

        public bool UpdateRelationshipAuthorizationsWith(long userId, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            return _friendshipQueryExecutor.UpdateRelationshipAuthorizationsWith(userId, retweetsEnabled, deviceNotifictionEnabled);
        }

        public bool UpdateRelationshipAuthorizationsWith(string userScreenName, bool retweetsEnabled, bool deviceNotifictionEnabled)
        {
            return _friendshipQueryExecutor.UpdateRelationshipAuthorizationsWith(userScreenName, retweetsEnabled, deviceNotifictionEnabled);
        }
    }
}
