﻿using System.Collections.Generic;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.Controllers;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviControllers.Timeline
{
    public class TimelineController : ITimelineController
    {
        private readonly ITweetFactory _tweetFactory;
        private readonly ITimelineQueryExecutor _timelineQueryExecutor;

        public TimelineController(
            ITweetFactory tweetFactory,
            ITimelineQueryExecutor timelineQueryExecutor)
        {
            _tweetFactory = tweetFactory;
            _timelineQueryExecutor = timelineQueryExecutor;
        }

        // Home Timeline
        public List<ITweet> GetHomeTimeline(int maximumTweets = 40, bool excludeReplies = false)
        {
            var timelineDTO = _timelineQueryExecutor.GetHomeTimeline(maximumTweets, excludeReplies);
            return _tweetFactory.GenerateTweetsFromDTO(timelineDTO);
        }

        // User Timeline
        public List<ITweet> GetUserTimeline(IUser user, int maximumTweets = 40, bool excludeReplies = false)
        {
            if (user == null)
            {
                return null;
            }

            return GetUserTimeline(user.UserDTO, maximumTweets, excludeReplies);
        }

        public List<ITweet> GetUserTimeline(IUserIdDTO userDTO, int maximumTweets = 40, bool excludeReplies = false)
        {
            var tweetsDTO = _timelineQueryExecutor.GetUserTimeline(userDTO, maximumTweets, excludeReplies);
            return _tweetFactory.GenerateTweetsFromDTO(tweetsDTO);
        }

        public List<ITweet> GetUserTimeline(long userId, int maximumTweets = 40, bool excludeReplies = false)
        {
            var tweetsDTO = _timelineQueryExecutor.GetUserTimeline(userId, maximumTweets, excludeReplies);
            return _tweetFactory.GenerateTweetsFromDTO(tweetsDTO);
        }

        public List<ITweet> GetUserTimeline(string userScreenName, int maximumTweets = 40, bool excludeReplies = false)
        {
            var tweetsDTO = _timelineQueryExecutor.GetUserTimeline(userScreenName, maximumTweets, excludeReplies);
            return _tweetFactory.GenerateTweetsFromDTO(tweetsDTO);
        }

        // Mention Timeline
        public List<IMention> GetMentionsTimeline(int maximumTweets = 40)
        {
            var timelineDTO = _timelineQueryExecutor.GetMentionsTimeline(maximumTweets);
            return _tweetFactory.GenerateMentionsFromDTO(timelineDTO);
        }
    }
}
