﻿using System;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.QueryGenerators;

namespace TweetinviControllers.User
{
    public class UserQueryGenerator : IUserQueryGenerator
    {
        public string GenerateUserIdParameter(long userId, string parameterName = "user_id")
        {
            return String.Format("{0}={1}", parameterName, userId);
        }

        public string GenerateScreenNameParameter(string userScreenName, string parameterName = "screen_name")
        {
            return String.Format("{0}={1}", parameterName, userScreenName);
        }

        public string GenerateIdOrScreenNameParameter(
            IUserIdDTO userDTO, 
            string idParameterName = "user_id",
            string screenNameParameterName = "screen_name")
        {
            if (userDTO == null)
            {
                throw new ArgumentException("Cannot extract id or name parameter from a null userDTO.");
            }

            if (userDTO.Id != -1)
            {
                return GenerateUserIdParameter(userDTO.Id, idParameterName);
            }

            if (!String.IsNullOrEmpty(userDTO.ScreenName))
            {
                return GenerateScreenNameParameter(userDTO.ScreenName, screenNameParameterName);
            }

            throw new ArgumentException("Cannot extract either id or name parameter from the given userDTO.");
        }
    }
}
