﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.Practices.Unity;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviFactories.Direct_Messages
{
    public class MessageFactory : IMessageFactory
    {
        private readonly IMessageFactoryQueryExecutor _messageFactoryQueryExecutor;
        private readonly IUnityFactory<IMessage> _messageUnityFactory;

        public MessageFactory(
            IMessageFactoryQueryExecutor messageFactoryQueryExecutor,
            IUnityFactory<IMessage> messageUnityFactory)
        {
            _messageFactoryQueryExecutor = messageFactoryQueryExecutor;
            _messageUnityFactory = messageUnityFactory;
        }

        // Get existing message
        public IMessage GetExistingMessage(long messageId)
        {
            var messageDTO = _messageFactoryQueryExecutor.GetExistingMessage(messageId);
            return GenerateMessageFromMessageDTO(messageDTO);
        }
        
        // Create Message
        public IMessage CreateMessage(string text, IUser recipient = null)
        {
            var messageDTO = _messageFactoryQueryExecutor.CreateMessage(text, recipient != null ? recipient.UserDTO : null);
            return GenerateMessageFromMessageDTO(messageDTO);
        }

        // Generate Message from DTO
        public IMessage GenerateMessageFromMessageDTO(IMessageDTO messageDTO)
        {
            return _messageUnityFactory.Create(new ResolverOverride[]
            {
                new ParameterOverride("messageDTO", messageDTO), 
            });
        }

        public List<IMessage> GenerateMessagesFromMessagesDTO(IEnumerable<IMessageDTO> messagesDTO)
        {
            if (messagesDTO == null)
            {
                return null;
            }

            return messagesDTO.Select(GenerateMessageFromMessageDTO).ToList();
        }
    }
}
