﻿using System.Collections.Generic;
using System.Linq;
using Microsoft.Practices.Unity;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;

namespace TweetinviFactories.Tweet
{
    public class TweetFactory : ITweetFactory
    {
        private readonly ITweetFactoryQueryExecutor _tweetDTOFactory;
        private readonly IUnityContainer _container;

        public TweetFactory(
            ITweetFactoryQueryExecutor tweetDTOFactory,
            IUnityContainer container)
        {
            _tweetDTOFactory = tweetDTOFactory;
            _container = container;
        }

        // Get Tweet
        public ITweet GetTweet(long tweetId)
        {
            var tweetDTO = _tweetDTOFactory.GetTweetDTO(tweetId);
            return GenerateTweetFromDTO(tweetDTO);
        }

        // Create Tweet
        public ITweet CreateTweet(string text)
        {
            var tweetDTO = _tweetDTOFactory.CreateTweetDTO(text);
            return GenerateTweetFromDTO(tweetDTO);
        }

        // Generate Tweet From DTO
        public ITweet GenerateTweetFromDTO(ITweetDTO tweetDTO)
        {
            if (tweetDTO == null)
            {
                return null;
            }

            var tweet = _container.Resolve<ITweet>(new ResolverOverride[]
            {
                new ParameterOverride("tweetDTO", tweetDTO)
            });

            return tweet;
        }

        public List<ITweet> GenerateTweetsFromDTO(List<ITweetDTO> tweetsDTO)
        {
            if (tweetsDTO == null)
            {
                return null;
            }

            List<ITweet> tweets = new List<ITweet>();

            for (int i = 0; i < tweetsDTO.Count; ++i)
            {
                var tweet = GenerateTweetFromDTO(tweetsDTO[i]);
                if (tweet != null)
                {
                    tweets.Add(tweet);
                }
            }

            return tweets;
        }

        // Generate OEmbedTweet from DTO
        public IOEmbedTweet GenerateOEmbedTweetDTO(IOEmbedTweetDTO oEmbedTweetDTO)
        {
            if (oEmbedTweetDTO == null)
            {
                return null;
            }

            var oEmbedTweet = _container.Resolve<IOEmbedTweet>(new ResolverOverride[]
            {
                new ParameterOverride("oEmbedTweetDTO", oEmbedTweetDTO)
            });

            return oEmbedTweet;
        }

        // Generate Mention from DTO
        public IMention GenerateMentionFromDTO(ITweetDTO tweetDTO)
        {
            if (tweetDTO == null)
            {
                return null;
            }

            var mention = _container.Resolve<IMention>(new ResolverOverride[]
            {
                new ParameterOverride("tweetDTO", tweetDTO)
            });

            return mention;
        }

        public List<IMention> GenerateMentionsFromDTO(IEnumerable<ITweetDTO> tweetsDTO)
        {
            if (tweetsDTO == null)
            {
                return null;
            }

            return tweetsDTO.Select(GenerateMentionFromDTO).ToList();
        }
    }
}
