﻿using System;
using Microsoft.Practices.Unity;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.TwitterToken;
using Tweetinvi.DTO;
using TweetinviFactories.Properties;

namespace TweetinviFactories.Tweet
{
    public interface ITweetFactoryQueryExecutor
    {
        ITweetDTO GetTweetDTO(long tweetId);
        ITweetDTO CreateTweetDTO(string text);
    }

    public class TweetFactoryQueryExecutor : ITweetFactoryQueryExecutor
    {
        private readonly ITwitterAccessor _twitterAccessor;
        private readonly IUnityContainer _container;

        public TweetFactoryQueryExecutor(
            ITwitterAccessor twitterAccessor,
            IUnityContainer container)
        {
            _twitterAccessor = twitterAccessor;
            _container = container;
        }

        public ITweetDTO GetTweetDTO(long tweetId)
        {
            string query = String.Format(Resources.Tweet_Get, tweetId);
            return _twitterAccessor.ExecuteGETQuery<TweetDTO>(query);
        }

        public ITweetDTO CreateTweetDTO(string text)
        {
            var tweetDTO = _container.Resolve<ITweetDTO>();
            tweetDTO.Text = text;

            return tweetDTO;
        }
    }
}
