﻿using System.Collections.Generic;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Unity;
using oAuthConnection;
using Tweetinvi;
using TwitterToken;

namespace TweetinviFactories
{
    public class TweetinviFactoriesBootstrapper
    {
        private static List<IModule> _moduleCatalog;

        private static IUnityContainer _container;
        public static IUnityContainer Container
        {
            get { return _container; }
        }

        static TweetinviFactoriesBootstrapper()
        {
            Initialize();
        }

        private static void Initialize()
        {
            _container = new UnityContainer();
            _moduleCatalog = new List<IModule>();

            RegisterModules();
            InitialiseModules();
        }

        private static void RegisterModules()
        {
            _moduleCatalog.Add(new OAuthConnectionModule(_container));
            _moduleCatalog.Add(new TwitterTokenModule(_container));
            _moduleCatalog.Add(new TweetinviFactoriesModule(_container));
            _moduleCatalog.Add(new TweetinviModule(_container));
        }

        private static void InitialiseModules()
        {
            foreach (var module in _moduleCatalog)
            {
                module.Initialize();
            }
        }
    }
}
