﻿using Microsoft.Practices.Unity;
using TweetinCore.Interfaces;
using TweetinCore.Interfaces.DTO;
using TweetinCore.Interfaces.Factories;
using Tweetinvi.DTO;
using Tweetinvi.TwitterEntities;
using TweetinviFactories.Direct_Messages;
using TweetinviFactories.Friendship;
using TweetinviFactories.Tweet;
using TweetinviFactories.User;

namespace TweetinviFactories
{
    public class TweetinviFactoriesModule : ITweetinviModule
    {
        private bool _isInitialized;
        private readonly IUnityContainer _container;

        public TweetinviFactoriesModule(IUnityContainer container)
        {
            _container = container;
            _isInitialized = false;
        }

        public IUnityContainer Container
        {
            get { return _container; }
        }

        public bool IsModuleInitialized()
        {
            return _isInitialized;
        }

        public void Initialize()
        {
            RegisterDTO();
            RegisterFactories();

            _container.RegisterType(typeof(IUnityFactory<>), typeof(UnityFactory<>));
            _isInitialized = true;
        }

        private void RegisterDTO()
        {
            _container.RegisterType<ITweetDTO, TweetDTO>();
            _container.RegisterType<IUserDTO, UserDTO>();
            _container.RegisterType<IMessageDTO, MessageDTO>();
            _container.RegisterType<IRelationshipDTO, RelationshipDTO>();

            _container.RegisterType<ITweetEntities, TweetEntities>();
            _container.RegisterType<IUrlEntity, UrlEntity>();
            _container.RegisterType<IHashtagEntity, HashtagEntity>();
        }

        private void RegisterFactories()
        {
            _container.RegisterType<ITweetFactory, TweetFactory>();
            _container.RegisterType<ITweetFactoryQueryExecutor, TweetFactoryQueryExecutor>();

            _container.RegisterType<IUserFactory, UserFactory>();
            _container.RegisterType<IUserFactoryQueryExecutor, UserFactoryQueryExecutor>();
            _container.RegisterType<IUserFactoryQueryGenerator, UserFactoryQueryGenerator>();

            _container.RegisterType<IFriendshipFactory, FriendshipFactory>();
            _container.RegisterType<IFriendshipFactoryQueryExecutor, FriendshipFactoryQueryExecutor>();

            _container.RegisterType<IMessageFactory, MessageFactory>();
            _container.RegisterType<IMessageFactoryQueryExecutor, MessageFactoryQueryExecutor>();
        }
    }
}
