﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TweetinCore.Interfaces.DTO;

namespace TweetinviFactories.User
{
    public interface IUserFactoryQueryGenerator
    {
        string GenerateListOfIdsParameter(List<long> ids);
        string GenerateListOfUserDTOParameter(List<IUserDTO> usersDTO);
        string GenerateListOfScreenNameParameter(List<string> userNames);
    }

    public class UserFactoryQueryGenerator : IUserFactoryQueryGenerator
    {
        public string GenerateListOfUserDTOParameter(List<IUserDTO> usersDTO)
        {
            if (usersDTO.Any(user => user.Id == -1 && String.IsNullOrEmpty(user.ScreenName)))
            {
                throw new ArgumentException("Cannot generate a list with any empty screename and id");
            }

            const string initialUserId = "user_id=";
            const string initialScreenName = "&screen_name=";

            StringBuilder idsBuilder = new StringBuilder(initialUserId);
            StringBuilder screeNameBuilder = new StringBuilder(initialScreenName);

            for (int i = 0; i < usersDTO.Count - 1; ++i)
            {
                var userDTO = usersDTO[0];

                if (userDTO.Id != -1)
                {
                    idsBuilder.Append(String.Format("{0}%2C", userDTO.Id));
                }
                else
                {
                    screeNameBuilder.Append(String.Format("{0}%2C", userDTO.ScreenName));
                }
            }

            // Last element does not have a comma
            if (usersDTO[usersDTO.Count - 1].Id != -1)
            {
                idsBuilder.Append(usersDTO[usersDTO.Count - 1].Id);
            }
            else
            {
                screeNameBuilder.Append(usersDTO[usersDTO.Count - 1].ScreenName);
            }

            // Only ids
            if (idsBuilder.ToString() == initialUserId)
            {
                return screeNameBuilder.ToString();
            }

            // Only screenames
            if (screeNameBuilder.ToString() == initialScreenName)
            {
                return idsBuilder.ToString();
            }

            // Both
            return idsBuilder.Append(screeNameBuilder).ToString();
        }

        public string GenerateListOfIdsParameter(List<long> ids)
        {
            StringBuilder builder = new StringBuilder();
            
            for (int i = 0; i < ids.Count - 1; ++i)
            {
                builder.Append(String.Format("{0}%2C", ids.ElementAt(i)));
            }

            builder.Append(ids.ElementAt(ids.Count - 1));

            return builder.ToString();
        }

        public string GenerateListOfScreenNameParameter(List<string> userNames)
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < userNames.Count - 1; ++i)
            {
                builder.Append(String.Format("{0}%2C", userNames.ElementAt(i)));
            }

            builder.Append(userNames.ElementAt(userNames.Count - 1));

            return builder.ToString();
        }
    }
}
